# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Convenience function for extracting the values for logging calls.

Because TensorFlow generally defers computation of values to a session run call,
it is impractical to log the values of tensors when they are defined. Instead,
the definition of a tensor is logged as normal using the log function in
mlperf_log.py and a tf.print statement helper function can be used to report
the relevant values as they are computed.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import uuid

import tensorflow as tf


def log_deferred(op, log_id, every_n=1, first_n=None):
  """Helper method inserting compliance logging ops.

  Note: This helper is not guaranteed to be efficient, as it will insert ops
        and control dependencies. If this proves to be a bottleneck, submitters
        may wish to consider other methods such as extracting values from an
        .events file.

  Args:
    op: A tf op to be printed.
    log_id: a uuid provided by the logger in mlperf_log.py
    every_n: If repeat is True, with what frequency should the input op be '
             logged. If repeat is False, this argument is ignored.
    first_n: Only log this many values. This arg does not interact with every_n.
             The first_n refers to the first n that would have been logged.
  """

  prefix = ":::MLPv0.5.0 [{}]".format(log_id)
  if not first_n is not None and first_n == 1:
    return tf.Print(op, [tf.timestamp(), op], message=prefix, first_n=1)

  counter = tf.Variable(tf.zeros(shape=(), dtype=tf.int32) - 1)
  increment = tf.assign_add(counter, 1, use_locking=True)
  return tf.cond(
      tf.equal(tf.mod(increment, every_n), 0),
      lambda :tf.Print(op, [tf.timestamp(), op], message=prefix,
                       first_n=first_n),
      lambda :op
  )


def sum_metric(tensor, name):
  sum_var = tf.Variable(
    initial_value=tf.zeros(shape=(), dtype=tensor.dtype),
    trainable=False,
    collections=[
      tf.GraphKeys.LOCAL_VARIABLES,
      tf.GraphKeys.METRIC_VARIABLES,
    ],
    name="{}_total".format(name)
  )

  update_op = tf.identity(tf.assign_add(sum_var, tensor))
  return tf.identity(sum_var, name=name), update_op


def _example():
  for kwargs in [dict(first_n=1), dict(), dict(every_n=2),
                 dict(first_n=2, every_n=2)]:
    op = tf.assign_add(tf.Variable(tf.zeros(shape=(), dtype=tf.int32) - 1), 1)
    op = log_deferred(op, str(uuid.uuid4()), **kwargs)
    init = [tf.local_variables_initializer(), tf.global_variables_initializer()]
    print("-" * 5)
    with tf.Session().as_default() as sess:
      sess.run(init)
      for _ in range(6):
        sess.run(op)


if __name__ == "__main__":
  _example()



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Keys which only appear in GNMT RNN Translation.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


# Loss smoothing factor
MODEL_HP_LOSS_SMOOTHING = "model_hp_loss_smoothing"

# Number of layers in encoder and in decoder
MODEL_HP_NUM_LAYERS = "model_hp_num_layers"

# RNN hidden size
MODEL_HP_HIDDEN_SIZE = "model_hp_hidden_size"

# Dropout
MODEL_HP_DROPOUT = "model_hp_dropout"

# Beam size for beam search
EVAL_HP_BEAM_SIZE = "eval_hp_beam_size"

# Maximum sequence length for training
TRAIN_HP_MAX_SEQ_LEN = "train_hp_max_sequence_length"

# Maximum sequence length for evaluation
EVAL_HP_MAX_SEQ_LEN = "eval_hp_max_sequence_length"

# Length normalization constant for beam search
EVAL_HP_LEN_NORM_CONST = "eval_hp_length_normalization_constant"

# Length normalization factor for beam search
EVAL_HP_LEN_NORM_FACTOR = "eval_hp_length_normalization_factor"

# Coverage penalty factor for beam search
EVAL_HP_COV_PENALTY_FACTOR = "eval_hp_coverage_penalty_factor"



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Keys which only appear in NCF Recommendation.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# The minimum number of ratings for a user to be included.
PREPROC_HP_MIN_RATINGS = "preproc_hp_min_ratings"

# The number of false negatives to use during evaluation.
PREPROC_HP_NUM_EVAL = "preproc_hp_num_eval"

# Are evaluation negatives sampled with replacement?
PREPROC_HP_SAMPLE_EVAL_REPLACEMENT = "preproc_hp_sample_eval_replacement"


# The number of false negatives per postive generated during training.
INPUT_HP_NUM_NEG = "input_hp_num_neg"

# Are training negatives sampled with replacement?
INPUT_HP_SAMPLE_TRAIN_REPLACEMENT = "input_hp_sample_train_replacement"

# This tag should be emitted each time the submission begins construction of the
# false negatives for a trainging epoch.
INPUT_STEP_TRAIN_NEG_GEN = "input_step_train_neg_gen"

# This tag should be emitted when the evaluation negatives are selected. This
# should occur only once.
INPUT_STEP_EVAL_NEG_GEN = "input_step_eval_neg_gen"

# The number of users in the evaluation set. This should be the same as the
# number of users in the training set.
EVAL_HP_NUM_USERS = "eval_hp_num_users"

# The number of false negatives per positive which actually appear during
# evaluation. This should match PREPROC_HP_NUM_EVAL.
EVAL_HP_NUM_NEG = "eval_hp_num_neg"


# The dimensionality of the matrix factorization portion of the model.
MODEL_HP_MF_DIM = "model_hp_mf_dim"

# The sizes of the fully connected layers in the dense section of the model.
MODEL_HP_MLP_LAYER_SIZES = "model_hp_mlp_layer_sizes"




# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Verification script to check model tag sets.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from collections import defaultdict
import re

from mlperf_compliance import _gnmt_tags
from mlperf_compliance import _ncf_tags
from mlperf_compliance import _resnet_tags
from mlperf_compliance import _ssd_tags
from mlperf_compliance import _transformer_tags
from mlperf_compliance  import tags


_MODEL_TAG_MODULES = [_gnmt_tags, _ncf_tags, _resnet_tags, _ssd_tags,
                      _transformer_tags]

TAG_PATTERN = re.compile("^[A-Za-z0-9_]+$")


def extract_tags(module):
  output = []
  for i in dir(module):
    if i.startswith("_") or not isinstance(getattr(module, i), str):
      continue
    output.append(i)
  return output


def check_collisions():
  defining_modules = defaultdict(list)
  for module in _MODEL_TAG_MODULES:
    name = module.__name__
    for tag in extract_tags(module):
      defining_modules[tag].append(name)

  duplicate_defs = {k: v for k, v in defining_modules.items() if len(v) > 1}
  for key in duplicate_defs.keys():
    print("Variable {} defined multiple times".format(key))


def check_format():
  for tag in sorted(tags.ALL_USED_TAGS):
    if not TAG_PATTERN.match(tag):
      print("Malformed tag: {}".format(tag))


if __name__ == "__main__":
  check_collisions()
  check_format()




# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Convenience function for logging compliance tags to stdout.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import inspect
import logging
import json
import os
import re
import sys
import time
import uuid

from mlperf_compliance.tags import *

ROOT_DIR_GNMT = None
ROOT_DIR_MASKRCNN = None
ROOT_DIR_MINIGO = None
ROOT_DIR_NCF = None

# Set by imagenet_main.py
ROOT_DIR_RESNET = None

ROOT_DIR_SSD = None

# Set by transformer_main.py and process_data.py
ROOT_DIR_TRANSFORMER = None


PATTERN = re.compile('[a-zA-Z0-9]+')

LOG_FILE = os.getenv("COMPLIANCE_FILE")
# create logger with 'spam_application'
LOGGER = logging.getLogger('mlperf_compliance')
LOGGER.setLevel(logging.DEBUG)

_STREAM_HANDLER = logging.StreamHandler(stream=sys.stdout)
_STREAM_HANDLER.setLevel(logging.INFO)
LOGGER.addHandler(_STREAM_HANDLER)

if LOG_FILE:
  _FILE_HANDLER = logging.FileHandler(LOG_FILE)
  _FILE_HANDLER.setLevel(logging.DEBUG)
  LOGGER.addHandler(_FILE_HANDLER)
else:
  _STREAM_HANDLER.setLevel(logging.DEBUG)



def get_caller(stack_index=2, root_dir=None):
  ''' Returns file.py:lineno of your caller. A stack_index of 2 will provide
      the caller of the function calling this function. Notice that stack_index
      of 2 or more will fail if called from global scope. '''
  caller = inspect.getframeinfo(inspect.stack()[stack_index][0])

  # Trim the filenames for readability.
  filename = caller.filename
  if root_dir is not None:
    filename = re.sub("^" + root_dir + "/", "", filename)
  return "%s:%d" % (filename, caller.lineno)


def _mlperf_print(key, value=None, benchmark=None, stack_offset=0,
                  tag_set=None, deferred=False, root_dir=None,
                  extra_print=False):
  ''' Prints out an MLPerf Log Line.

  key: The MLPerf log key such as 'CLOCK' or 'QUALITY'. See the list of log keys in the spec.
  value: The value which contains no newlines.
  benchmark: The short code for the benchmark being run, see the MLPerf log spec.
  stack_offset: Increase the value to go deeper into the stack to find the callsite. For example, if this
                is being called by a wraper/helper you may want to set stack_offset=1 to use the callsite
                of the wraper/helper itself.
  tag_set: The set of tags in which key must belong.
  deferred: The value is not presently known. In that case, a unique ID will
            be assigned as the value of this call and will be returned. The
            caller can then include said unique ID when the value is known
            later.
  root_dir: Directory prefix which will be trimmed when reporting calling file
            for compliance logging.
  extra_print: Print a blank line before logging to clear any text in the line.

  Example output:
    :::MLP-1537375353 MINGO[17] (eval.py:42) QUALITY: 43.7
  '''

  return_value = None

  if (tag_set is None and not PATTERN.match(key)) or key not in tag_set:
    raise ValueError('Invalid key for MLPerf print: ' + str(key))

  if value is not None and deferred:
    raise ValueError("deferred is set to True, but a value was provided")

  if deferred:
    return_value = str(uuid.uuid4())
    value = "DEFERRED: {}".format(return_value)

  if value is None:
    tag = key
  else:
    str_json = json.dumps(value)
    tag = '{key}: {value}'.format(key=key, value=str_json)

  callsite = get_caller(2 + stack_offset, root_dir=root_dir)
  now = time.time()

  message = ':::MLPv0.5.0 {benchmark} {secs:.9f} ({callsite}) {tag}'.format(
      secs=now, benchmark=benchmark, callsite=callsite, tag=tag)

  if extra_print:
    print() # There could be prior text on a line

  if tag in STDOUT_TAG_SET:
    LOGGER.info(message)
  else:
    LOGGER.debug(message)

  return return_value


GNMT_TAG_SET = set(GNMT_TAGS)
def gnmt_print(key, value=None, stack_offset=1, deferred=False):
  return _mlperf_print(key=key, value=value, benchmark=GNMT,
                       stack_offset=stack_offset, tag_set=GNMT_TAG_SET,
                       deferred=deferred, root_dir=ROOT_DIR_GNMT)


MASKRCNN_TAG_SET = set(MASKRCNN_TAGS)
def maskrcnn_print(key, value=None, stack_offset=1, deferred=False,
    extra_print=True):
  return _mlperf_print(key=key, value=value, benchmark=MASKRCNN,
                       stack_offset=stack_offset, tag_set=MASKRCNN_TAG_SET,
                       deferred=deferred, extra_print=extra_print,
                       root_dir=ROOT_DIR_MASKRCNN)


MINIGO_TAG_SET = set(MINIGO_TAGS)
def minigo_print(key, value=None, stack_offset=1, deferred=False):
  return _mlperf_print(key=key, value=value, benchmark=MINIGO,
                       stack_offset=stack_offset, tag_set=MINIGO_TAG_SET,
                       deferred=deferred, root_dir=ROOT_DIR_MINIGO)


NCF_TAG_SET = set(NCF_TAGS)
def ncf_print(key, value=None, stack_offset=1, deferred=False,
              extra_print=True):
  # Extra print is needed for the reference NCF because of tqdm.
  return _mlperf_print(key=key, value=value, benchmark=NCF,
                       stack_offset=stack_offset, tag_set=NCF_TAG_SET,
                       deferred=deferred, extra_print=extra_print,
                       root_dir=ROOT_DIR_NCF)


RESNET_TAG_SET = set(RESNET_TAGS)
def resnet_print(key, value=None, stack_offset=1, deferred=False):
  return _mlperf_print(key=key, value=value, benchmark=RESNET,
                       stack_offset=stack_offset, tag_set=RESNET_TAG_SET,
                       deferred=deferred, root_dir=ROOT_DIR_RESNET)


SSD_TAG_SET = set(SSD_TAGS)
def ssd_print(key, value=None, stack_offset=1, deferred=False,
              extra_print=True):
  return _mlperf_print(key=key, value=value, benchmark=SSD,
                       stack_offset=stack_offset, tag_set=SSD_TAG_SET,
                       deferred=deferred, extra_print=extra_print,
                       root_dir=ROOT_DIR_SSD)


TRANSFORMER_TAG_SET = set(TRANSFORMER_TAGS)
def transformer_print(key, value=None, stack_offset=1, deferred=False):
  return _mlperf_print(key=key, value=value, benchmark=TRANSFORMER,
                       stack_offset=stack_offset, tag_set=TRANSFORMER_TAG_SET,
                       deferred=deferred, root_dir=ROOT_DIR_TRANSFORMER)


if __name__ == '__main__':
  ncf_print(EVAL_ACCURACY, {'epoch': 7, 'accuracy': 43.7})
  ncf_print(INPUT_SIZE, 1024)



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Keys which only appear in SSD.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


# Pretrained classifer model
BACKBONE = "backbone"

FEATURE_SIZES = "feature_sizes"
STEPS = "steps"
SCALES = "scales"
ASPECT_RATIOS = "aspect_ratios"
NUM_DEFAULTS_PER_CELL = "num_defaults_per_cell"
LOC_CONF_OUT_CHANNELS = "loc_conf_out_channels"
NUM_DEFAULTS = "num_default_boxes"

# Overlap threshold for NMS
NMS_THRESHOLD = "nms_threshold"
NMS_MAX_DETECTIONS = "nms_max_detections"

# data pipeline
NUM_CROPPING_ITERATIONS = "num_cropping_iterations"
RANDOM_FLIP_PROBABILITY = "random_flip_probability"
DATA_NORMALIZATION_MEAN = "data_normalization_mean"
DATA_NORMALIZATION_STD = "data_normalization_std"



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Convenience functions for logging ResNet topology.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from mlperf_compliance import mlperf_log

_STACK_OFFSET = 2

def _get_shape(input_tensor):
  return "({})".format(", ".join(
      [str(i) for i in input_tensor.shape.as_list()[1:]]))


def _in_out_shape(input_tensor, output_tensor):
  return "{} -> {}".format( _get_shape(input_tensor), _get_shape(output_tensor))


def log_max_pool(input_tensor, output_tensor):
  mlperf_log.resnet_print(
      key=mlperf_log.MODEL_HP_INITIAL_MAX_POOL, value=_in_out_shape(
      input_tensor=input_tensor, output_tensor=output_tensor),
      stack_offset=_STACK_OFFSET)


def log_begin_block(input_tensor, block_type):
  mlperf_log.resnet_print(key=mlperf_log.MODEL_HP_BEGIN_BLOCK,
                          value={"block_type": block_type},
                          stack_offset=_STACK_OFFSET)
  mlperf_log.resnet_print(
      key=mlperf_log.MODEL_HP_RESNET_TOPOLOGY,
      value=" Block Input: {}".format(_get_shape(input_tensor)),
      stack_offset=_STACK_OFFSET)


def log_end_block(output_tensor):
  mlperf_log.resnet_print(
      key=mlperf_log.MODEL_HP_END_BLOCK,
      value=" Block Output: {}".format(_get_shape(output_tensor)),
      stack_offset=_STACK_OFFSET)


def log_projection(input_tensor, output_tensor):
  mlperf_log.resnet_print(
      key=mlperf_log.MODEL_HP_PROJECTION_SHORTCUT,
      value=_in_out_shape(input_tensor, output_tensor),
      stack_offset=_STACK_OFFSET)


def log_conv2d(input_tensor, output_tensor, stride, filters, initializer,
               use_bias):
  mlperf_log.resnet_print(key=mlperf_log.MODEL_HP_CONV2D_FIXED_PADDING,
                          value=_in_out_shape(input_tensor, output_tensor),
                          stack_offset=_STACK_OFFSET)
  mlperf_log.resnet_print(
      key=mlperf_log.MODEL_HP_CONV2D_FIXED_PADDING,
      value={"stride": stride, "filters": filters, "initializer": initializer,
             "use_bias": use_bias},
      stack_offset=_STACK_OFFSET)


def log_batch_norm(input_tensor, output_tensor, momentum, epsilon, center,
                   scale, training):
  assert _get_shape(input_tensor) == _get_shape(output_tensor)
  mlperf_log.resnet_print(key=mlperf_log.MODEL_HP_BATCH_NORM, value={
    "shape": _get_shape(input_tensor), "momentum": momentum, "epsilon": epsilon,
    "center": center, "scale": scale, "training": training},
                          stack_offset=_STACK_OFFSET)



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

import setuptools

with open("README.md", "r") as f:
  long_description = f.read()

setuptools.setup(
    name="mlperf_compliance",
    version="0.0.7",
    author="Taylor Robie",
    author_email="taylorrobie@google.com",
    description="Tools for logging MLPerf compliance tags.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mlperf/training/tree/master/compliance",
    packages=['mlperf_compliance'],
    classifiers=[
      "Programming Language :: Python :: 2",
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: Apache Software License",
      "Operating System :: OS Independent",
    ],
    license='Apache 2.0',
)



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Master list of MLPerf tags to be logged for benchmark submissions.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from mlperf_compliance._gnmt_tags import *
from mlperf_compliance._ncf_tags import *
from mlperf_compliance._resnet_tags import *
from mlperf_compliance._ssd_tags import *
from mlperf_compliance._transformer_tags import *


# ==============================================================================
# == Benchmarks ================================================================
# ==============================================================================

# rnn_translator
GNMT = "gnmt"

# reinforcement/
MINIGO = "minigo"

# recommendation/
NCF = "ncf"

# image_classification/
RESNET = "resnet"

# single_stage_detector/
SSD = "ssd"

# object_detection/
MASKRCNN = "maskrcnn"

# translation/
TRANSFORMER = "transformer"

# ==============================================================================
# == Tags ======================================================================
# ==============================================================================
"""
Tags may be used by all models, a subset of models, or only one model. A
specification for which models require which tags can be found below the tag
definitions.
"""

# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# All models: Tags which should appear in absolutely every MLPerf model.
# //////////////////////////////////////////////////////////////////////////////

# This tag signals to start the timer. Emission of this tag need not be (and
# generally will not be) the first part of a submission script. Rather, this
# tag must be emitted prior to performing any work which the MLPerf rules
# state must be timed. This tag is generally emitted directly before the first
# step which invokes random number generation or the first step which must be
# performed on the system under test. (Whichever comes first.) If clarification
# is needed, please file an issue under:
#   https://github.com/mlperf/policies
RUN_START = "run_start"

# This tag signals that a submission has reached the relevant stopping criteria,
# and has completed all tasks which are performed in the reference. The wall
# time for a submission will be computed as the difference between the time
# when this tag is emitted and the time whe the RUN_START is emitted.
RUN_STOP = "run_stop"

# This tag should be emitted immediately before ending a run, and should be the
# last tag emitted. This tag should indicate the completion of untimed post
# processing work such as system specific cleanup.
RUN_FINAL = "run_final"


# Emit this tag in the place(s) where random seeds are set.
RUN_SET_RANDOM_SEED = "run_set_random_seed"


# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# Common Values: Constants which are expected to be reported across many models.
#                These values are included for convenience.
# //////////////////////////////////////////////////////////////////////////////
BCE = "binary_cross_entropy"
CCE = "categorical_cross_entropy"

SGD = "stochastic_gradient_descent"

# Some conventions distinguish between "vanilla" SGD and SGD with momentum
# (where vanilla SGD would be the specific case of momentum=0)
SGD_WITH_MOMENTUM = "stochastic_gradient_descent_with_momentum"

ADAM = "adam"
LAZY_ADAM = "lazy_adam"

TRUNCATED_NORMAL = "truncated_normal"

RELU = "relu"


# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# Preprocessing: Tags for generic preprocessing steps
# //////////////////////////////////////////////////////////////////////////////

# The number of training examples in a single epoch
PREPROC_NUM_TRAIN_EXAMPLES = "preproc_num_train_examples"

# The number of evaluation examples in a single epoch
PREPROC_NUM_EVAL_EXAMPLES = "preproc_num_eval_examples"

# This tag is used to declare what part of code tokenizes the training data.
PREPROC_TOKENIZE_TRAINING = "preproc_tokenize_training"

# This tag is used to declare what part of code tokenizes the evaluation data.
PREPROC_TOKENIZE_EVAL = "preproc_tokenize_eval"

# The vocabulary size used for tokenization.
PREPROC_VOCAB_SIZE = "preproc_vocab_size"


# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# Input: Tags for the timed portion of the data input pipeline
# //////////////////////////////////////////////////////////////////////////////

# The number of examples in the training portion of the data pipeline. Generally
# this should match PREPROC_NUM_TRAIN_EXAMPLES. If it does not (for instance
# if certain examples are dropped in compliance with MLPerf rules), the
# call which declares this tag is a good place for a comment stating why the
# disparity is expected.
INPUT_SIZE = "input_size"

# The size of a training minibatch size. If this value is variable, please emit
# "-1" and then log an implementation specific characterization of the batch
# size which is a reasonable analog to the reference. (For instance log that
# all but the last batch has size 64, and the last batch is a partial batch)
INPUT_BATCH_SIZE = "input_batch_size"

# This tag indicates where the location of the code which defines the order in
# which training examples are traversed. It is not necessary to describe the
# method in the tag emission (though comments are always welcome). Rather, this
# should simply provide a good starting point to an interested party.
INPUT_ORDER = "input_order"

# The shard size (in items) when shuffling in the input pipeline.
INPUT_SHARD = "input_shard"


# --------------------------------------
# -- Data Augmentation and Alteration --
# --------------------------------------

# ResNet random cropping
INPUT_CENTRAL_CROP = "input_central_crop"

INPUT_CROP_USES_BBOXES = "input_crop_uses_bboxes"

INPUT_DISTORTED_CROP_MIN_OBJ_COV = "input_distorted_crop_min_object_covered"
INPUT_DISTORTED_CROP_RATIO_RANGE = "input_distorted_crop_aspect_ratio_range"
INPUT_DISTORTED_CROP_AREA_RANGE = "input_distorted_crop_area_range"
INPUT_DISTORTED_CROP_MAX_ATTEMPTS = "input_distorted_crop_max_attempts"

INPUT_MEAN_SUBTRACTION = "input_mean_subtraction"

# Random flip of an image for data augmentation
INPUT_RANDOM_FLIP = "input_random_flip"

INPUT_RESIZE = "input_resize"
INPUT_RESIZE_ASPECT_PRESERVING = "input_resize_aspect_preserving"


# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
# Opt: Tags for declaring optimizer specific information. Submissions should
#      declare and log explicit values rather than relying on defaults.
# //////////////////////////////////////////////////////////////////////////////

# The name of the optimizer used. (SGD, Adam, etc.)
OPT_NAME = "opt_name"

OPT_LR = "opt_learning_rate"
OPT_MOMENTUM = "opt_momentum"

OPT_WEIGHT_DECAY = "opt_weight_decay"

# beta1, beta2, and epsilon are optimizer hyperparameters associated with the
# Adam optimizer and its variants (e.g. LazyAdam).
OPT_HP_ADAM_BETA1 = "opt_hp_Adam_beta1"
OPT_HP_ADAM_BETA2 = "opt_hp_Adam_beta2"
OPT_HP_ADAM_EPSILON = "opt_hp_Adam_epsilon"


# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
#  Train: Tags for control flow during model training.
# //////////////////////////////////////////////////////////////////////////////

# This tag is emitted when a model first enters its training loop. This is not
# necessarily when it begins to apply gradients; rather, it should be placed at
# a location which logically partitions the submission code.
TRAIN_LOOP = "train_loop"

# The current epoch as said epoch begins training.
TRAIN_EPOCH = "train_epoch"

# This tag is used to indicate approximately where checkpoints are written. Some
# frameworks abstract away checkpoint saving; in such cases simply choose a
# logical place in the code which signals that the framework has been instructed
# to save checkpoints, along with an explanatory comment.
TRAIN_CHECKPOINT = "train_checkpoint"


# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
#  Eval: Tags for control flow during model evaluation.
# //////////////////////////////////////////////////////////////////////////////

# This tag should be emitted whenever the submission begins an evaluation pass
# for a given set of weights.
EVAL_START = "eval_start"

# The number of examples on which evaluation is performed.
EVAL_SIZE = "eval_size"

# The target quality at which the model may stop training.
EVAL_TARGET = "eval_target"

# The observed accuracy of the model at a given epoch.
EVAL_ACCURACY = "eval_accuracy"

# This tag should be emitted when the model has determined that it has met the
# target quality set by the reference.
EVAL_STOP = "eval_stop"


# The observed accuracy of the model at a given iteration. This is only for
# models which evaluate at certain iterations instead of epochs.
EVAL_ITERATION_ACCURACY = "eval_iteration_accuracy"

# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
#  Model: Tags for logging topology specific information.
# //////////////////////////////////////////////////////////////////////////////

# The loss function (cross entropy, squared error, etc.) used by the model. For
# more exotic loss functions such as those encountered in object detection
# models, additional benchmark specific subcomponents should also be logged.
MODEL_HP_LOSS_FN = "model_hp_loss_fn"

MODEL_HP_INITIAL_SHAPE = "model_hp_initial_shape"
MODEL_HP_FINAL_SHAPE = "model_hp_final_shape"

MODEL_L2_REGULARIZATION = "model_l2_regularization"
MODEL_EXCLUDE_BN_FROM_L2 = "model_exclude_bn_from_l2"

MODEL_HP_RELU = "model_hp_relu"
MODEL_HP_CONV2D_FIXED_PADDING = "model_hp_conv2d_fixed_padding"
MODEL_HP_BATCH_NORM = "model_hp_batch_norm"
MODEL_HP_DENSE = "model_hp_dense"


# ==============================================================================
# == Stdout tags ===============================================================
# ==============================================================================

# These tags are always logged to stdout. The rest will be logged to a file if
# one is available.
STDOUT_TAG_SET = {
    RUN_START,
    RUN_STOP,
    RUN_FINAL,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
}


# ==============================================================================
# == Benchmark tag sets ========================================================
# ==============================================================================
ALL_USED_TAGS = set()

GNMT_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,
    RUN_SET_RANDOM_SEED,

    PREPROC_VOCAB_SIZE,
    PREPROC_TOKENIZE_TRAINING,
    PREPROC_TOKENIZE_EVAL,
    PREPROC_NUM_TRAIN_EXAMPLES,
    PREPROC_NUM_EVAL_EXAMPLES,

    INPUT_SIZE,
    INPUT_BATCH_SIZE,
    INPUT_ORDER,
    INPUT_SHARD,

    OPT_NAME,
    OPT_LR,
    OPT_HP_ADAM_BETA1,
    OPT_HP_ADAM_BETA2,
    OPT_HP_ADAM_EPSILON,

    TRAIN_LOOP,
    TRAIN_EPOCH,
    TRAIN_CHECKPOINT,
    TRAIN_HP_MAX_SEQ_LEN,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
    EVAL_HP_BEAM_SIZE,
    EVAL_HP_MAX_SEQ_LEN,
    EVAL_HP_LEN_NORM_CONST,
    EVAL_HP_LEN_NORM_FACTOR,
    EVAL_HP_COV_PENALTY_FACTOR,

    MODEL_HP_LOSS_FN,
    MODEL_HP_LOSS_SMOOTHING,
    MODEL_HP_NUM_LAYERS,
    MODEL_HP_HIDDEN_SIZE,
    MODEL_HP_DROPOUT
)

MASKRCNN_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,

    INPUT_BATCH_SIZE,
    INPUT_ORDER,
    INPUT_SHARD,

    BACKBONE,
    NMS_THRESHOLD,
    NMS_MAX_DETECTIONS,

    OPT_NAME,
    OPT_LR,
    OPT_MOMENTUM,
    OPT_WEIGHT_DECAY,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
)

MINIGO_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,
    RUN_SET_RANDOM_SEED,

    INPUT_SHARD,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
)

NCF_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,

    PREPROC_HP_MIN_RATINGS,
    PREPROC_HP_NUM_EVAL,
    PREPROC_HP_SAMPLE_EVAL_REPLACEMENT,

    INPUT_SIZE,
    INPUT_BATCH_SIZE,
    INPUT_ORDER,
    INPUT_SHARD,
    INPUT_HP_NUM_NEG,
    INPUT_HP_SAMPLE_TRAIN_REPLACEMENT,
    INPUT_STEP_TRAIN_NEG_GEN,
    INPUT_STEP_EVAL_NEG_GEN,

    OPT_NAME,
    OPT_LR,
    OPT_HP_ADAM_BETA1,
    OPT_HP_ADAM_BETA2,
    OPT_HP_ADAM_EPSILON,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
    EVAL_HP_NUM_USERS,
    EVAL_HP_NUM_NEG,

    MODEL_HP_LOSS_FN,
    MODEL_HP_MF_DIM,
    MODEL_HP_MLP_LAYER_SIZES,
)

RESNET_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,
    RUN_SET_RANDOM_SEED,

    PREPROC_NUM_TRAIN_EXAMPLES,
    PREPROC_NUM_EVAL_EXAMPLES,

    INPUT_SIZE,
    INPUT_BATCH_SIZE,
    INPUT_ORDER,
    INPUT_SHARD,
    INPUT_CENTRAL_CROP,
    INPUT_CROP_USES_BBOXES,
    INPUT_DISTORTED_CROP_MIN_OBJ_COV,
    INPUT_DISTORTED_CROP_RATIO_RANGE,
    INPUT_DISTORTED_CROP_AREA_RANGE,
    INPUT_DISTORTED_CROP_MAX_ATTEMPTS,
    INPUT_MEAN_SUBTRACTION,
    INPUT_RANDOM_FLIP,
    INPUT_RESIZE,
    INPUT_RESIZE_ASPECT_PRESERVING,

    OPT_NAME,
    OPT_LR,
    OPT_MOMENTUM,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
    EVAL_EPOCH_OFFSET,

    MODEL_HP_LOSS_FN,
    MODEL_L2_REGULARIZATION,
    MODEL_EXCLUDE_BN_FROM_L2,

    MODEL_HP_INITIAL_SHAPE,
    MODEL_HP_FINAL_SHAPE,
    MODEL_HP_INITIAL_MAX_POOL,
    MODEL_HP_BEGIN_BLOCK,
    MODEL_HP_END_BLOCK,
    MODEL_HP_BLOCK_TYPE,
    MODEL_HP_PROJECTION_SHORTCUT,
    MODEL_HP_SHORTCUT_ADD,
    MODEL_HP_RELU,
    MODEL_HP_CONV2D_FIXED_PADDING,
    MODEL_HP_BATCH_NORM,
    MODEL_HP_DENSE,
    MODEL_HP_RESNET_TOPOLOGY,
)

SSD_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,

    INPUT_SIZE,
    INPUT_BATCH_SIZE,
    INPUT_ORDER,
    INPUT_SHARD,

    BACKBONE,
    FEATURE_SIZES,
    STEPS,
    SCALES,
    ASPECT_RATIOS,
    NUM_DEFAULTS_PER_CELL,
    LOC_CONF_OUT_CHANNELS,
    NUM_DEFAULTS,
    NMS_THRESHOLD,
    NMS_MAX_DETECTIONS,

    NUM_CROPPING_ITERATIONS,
    RANDOM_FLIP_PROBABILITY,
    DATA_NORMALIZATION_MEAN,
    DATA_NORMALIZATION_STD,

    OPT_NAME,
    OPT_LR,
    OPT_MOMENTUM,
    OPT_WEIGHT_DECAY,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_SIZE,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,
    EVAL_ITERATION_ACCURACY,
)

TRANSFORMER_TAGS = (
    RUN_START,
    RUN_STOP,
    RUN_FINAL,
    RUN_SET_RANDOM_SEED,

    PREPROC_NUM_TRAIN_EXAMPLES,
    PREPROC_NUM_EVAL_EXAMPLES,
    PREPROC_TOKENIZE_TRAINING,
    PREPROC_TOKENIZE_EVAL,
    PREPROC_VOCAB_SIZE,

    INPUT_BATCH_SIZE,
    INPUT_MAX_LENGTH,
    INPUT_ORDER,
    INPUT_SHARD,

    OPT_NAME,
    OPT_LR,
    OPT_LR_WARMUP_STEPS,
    OPT_HP_ADAM_BETA1,
    OPT_HP_ADAM_BETA2,
    OPT_HP_ADAM_EPSILON,

    TRAIN_LOOP,
    TRAIN_EPOCH,

    EVAL_START,
    EVAL_TARGET,
    EVAL_ACCURACY,
    EVAL_STOP,

    MODEL_HP_INITIALIZER_GAIN,
    MODEL_HP_VOCAB_SIZE,
    MODEL_HP_NUM_HIDDEN_LAYERS,
    MODEL_HP_EMBEDDING_SHARED_WEIGHTS,
    MODEL_HP_ATTENTION_DENSE,
    MODEL_HP_ATTENTION_DROPOUT,
    MODEL_HP_FFN_OUTPUT_DENSE,
    MODEL_HP_FFN_FILTER_DENSE,
    MODEL_HP_RELU_DROPOUT,
    MODEL_HP_LAYER_POSTPROCESS_DROPOUT,
    MODEL_HP_NORM,
    MODEL_HP_SEQ_BEAM_SEARCH,
)

ALL_USED_TAGS.update(GNMT_TAGS)
ALL_USED_TAGS.update(MASKRCNN_TAGS)
ALL_USED_TAGS.update(NCF_TAGS)
ALL_USED_TAGS.update(RESNET_TAGS)
ALL_USED_TAGS.update(SSD_TAGS)
ALL_USED_TAGS.update(TRANSFORMER_TAGS)



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Keys which only appear in ResNet.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


BOTTLENECK_BLOCK = "bottleneck_block"

# The ResNet reference specifies that evaluation occurs once every four epochs.
# This can result in a quantization penalty for batch sizes which converge on
# certain epochs. For instance a batch size which tends to converge on epoch 81
# or 82 would be unduly punished by evaluating at epochs 80 and 84. In order to
# address this, submissions may select an offset between 0 and 3 for the first
# evaluation. So in the example above, the submitter could select an offset of
# 1. In that case the first evaluation would occur on epoch 2, with later
# evaluations correspondingly offset. Because this would trigger an eval on
# epoch 82, the submission in this example can exit at a natural time.
EVAL_EPOCH_OFFSET = "eval_offset"

# ==============================================================================
# == Topology ==================================================================
# ==============================================================================

MODEL_HP_INITIAL_MAX_POOL = "model_hp_initial_max_pool"
MODEL_HP_BEGIN_BLOCK = "model_hp_begin_block"
MODEL_HP_END_BLOCK = "model_hp_end_block"
MODEL_HP_BLOCK_TYPE = "model_hp_block_type"
MODEL_HP_PROJECTION_SHORTCUT = "model_hp_projection_shortcut"
MODEL_HP_SHORTCUT_ADD = "model_hp_shorcut_add"

MODEL_HP_RESNET_TOPOLOGY = "model_hp_resnet_topology"



# Copyright 2018 MLBenchmark Group. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Keys which only appear in transformer.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

INPUT_MAX_LENGTH = "input_max_length"

OPT_LR_WARMUP_STEPS = "opt_learning_rate_warmup_steps"

MODEL_HP_INITIALIZER_GAIN = "model_hp_initializer_gain"
MODEL_HP_VOCAB_SIZE = "model_hp_vocab_size"
MODEL_HP_NUM_HIDDEN_LAYERS = "model_hp_hidden_layers"
MODEL_HP_EMBEDDING_SHARED_WEIGHTS = "model_hp_embedding_shared_weights"
MODEL_HP_ATTENTION_DENSE = "model_hp_attention_dense"
MODEL_HP_ATTENTION_DROPOUT = "model_hp_attention_dropout"
MODEL_HP_FFN_OUTPUT_DENSE = "model_hp_ffn_output_dense"
MODEL_HP_FFN_FILTER_DENSE = "model_hp_ffn_filter_dense"
MODEL_HP_RELU_DROPOUT = "model_hp_relu_dropout"
MODEL_HP_LAYER_POSTPROCESS_DROPOUT = "model_hp_layer_postprocess_dropout"
MODEL_HP_NORM = "model_hp_norm"
MODEL_HP_SEQ_BEAM_SEARCH = "model_hp_sequence_beam_search"
